package com.steerinfo.dil.mapper;

import com.steerinfo.dil.model.BmstruckDetailsOrder;
import com.steerinfo.framework.mapper.IBaseMapper;
import java.math.*;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

@Mapper
public interface BmstruckDetailsOrderMapper extends IBaseMapper<BmstruckDetailsOrder, BigDecimal> {

    /**
     * 展示汽运详单
     * @param mapVal
     * @return
     */
    List<Map<String,Object>> getTruckDetailsOrderList(Map<String,Object> mapVal);

    /**
     * 新增汽运详单
     * @param bmstruckDetailsOrder
     * @return
     */
    int insertSelective(BmstruckDetailsOrder bmstruckDetailsOrder);

    /**
     * 查询最大id
     * @return
     */
    BigDecimal selectMaxId();

    /**
     * 运单id
     * @param orderId
     * @return
     */
    BigDecimal getOrderType(BigDecimal orderId);

    /**
     * 展示汽运实绩列表
     * @param mapVal
     * @return
     */
    List<Map<String, Object>> getTruckResultList(Map<String, Object> mapVal);

    /**
     * 查询计算因子
     * @param orderId
     * @return
     */
    BigDecimal getFormulaMembers(BigDecimal orderId);

    /**
     * 得到价格id
     * @return
     */
    Map<String, Object> getPriceId(BigDecimal orderId);

    /**
     * 得到价格
     * @return
     */
    BigDecimal getPriceValue(BigDecimal priceId);

    /**
     * 得到到厂湿重
     * @param weightTaskResultId
     * @return
     */
    BigDecimal getNetWeight(BigDecimal weightTaskResultId);

//    /**
//     * 生成汽运账单
//     * @param mapList
//     * @return
//     */
//    int addTruckStatement(List<Map<String, Object>> mapList);

    @Select("select seq_BMSTRUCK_DETAIL.nextval from dual")
    BigDecimal selectDetailsId();

    /**
     * 展示汽运实绩列表
     * @param mapVal
     * @return
     */
    List<Map<String, Object>> getSaleTruckResultList(Map<String,Object> mapVal);

    /**
     * 查看未转账单的详单
     * @param mapVal
     * @return
     */
    List<Map<String, Object>> getUnFinishedTruckDetailsOrderList(Map<String, Object> mapVal);

    /**
     * 查看详单
     * @param mapVal
     * @return
     */
    List<Map<String, Object>> getSaleTruckDetailsOrderList(Map<String, Object> mapVal);

    //计时
    List<Map<String, Object>> getUnFinishedTimeTruckDetailsOrderList(Map<String, Object> mapVal);
    //计数
    List<Map<String, Object>> getUnFinishedNumTruckDetailsOrderList(Map<String, Object> mapVal);
    //计重
    List<Map<String, Object>> getUnFinishedWeightTruckDetailsOrderList(Map<String, Object> mapVal);
    //包月
    List<Map<String, Object>> getUnFinishedMonthTruckDetailsOrderList(Map<String, Object> mapVal);

    //通过运单id获取需求类型（需求类型（0：计重；1：计时；2：计次；3：包月））
    Map<String, Object> getSettlement(BigDecimal orderId);

    //通过物资id与需求类型获取单价
    BigDecimal getPriceByWz(String requirementType, Integer materialId);

    BigDecimal getCarrierIdBySSO(String carrierSsoId);

    Map<String,Object> findAddressAndPrice(BigDecimal priceId);

    BigDecimal getOrderId(BigDecimal weightTaskResultId);

    List<Map<String,Object>> getWeightTaskResultId(BigDecimal orderId);

    //内转计重结算
    List<Map<String, Object>> getInwardWeightDetailsOrder(Map<String, Object> mapValue);

    //采购内转计重结算
    List<Map<String, Object>> getPurInwarWeightDetailsOrder(Map<String, Object> mapValue);

    //内转计时结算
    List<Map<String, Object>> getInwardTimeDetailsOrder(Map<String, Object> mapValue);

    Map<String,Object> getPriceByPur(int i);

    //获取计时时长
    List<Map<String, Object>> getTimeTaskResult(BigDecimal orderId);

    List<Map<String, Object>> updateDetails();

    List<Map<String, Object>> selectDetailsNoAmount();

    List<Map<String, Object>> getNoInwardDetails(Map<String, Object> mapValue);

    Map<String, Object> getPriceInfoPur(BigDecimal orderId);

    Map<String, Object> getPurInwardPrice(Map<String, Object> priceInfoPur);

    List<Map<String, Object>> getAllInwardWeight(Map<String, Object> mapValue);

    List<Map<String, Object>> getSteelTruckDetailsOrder(Map<String, Object> mapValue);

    /**
     * @author:zyf
     * @version:1.0
     * @Date:
     * @Description: 查找出priceID
    */
    Map<String,Object> queryaddress(BigDecimal s);
    /**
     * @author:zyf
     * @version:1.0
     * @Date:
     * @Description:查询出收货地址
    */
    String queryaddress1(BigDecimal priceid);
    Integer selectDetailsByOrder(BigDecimal orderId);

    List<BigDecimal> getDetailsIdByOrderId(Integer orderId);

    BigDecimal selectNetWeight(BigDecimal orderId);
    //内转计次结算
    List<Map<String, Object>> getInwardCountDetailsOrder(Map<String, Object> mapValue);
    //查询该订单是否是计重且计时
    String selectWeightAndTimeByOrderId(BigDecimal orderId);

    int getIsHaveDetailsOrder(BigDecimal weightTaskResultId);

    int getIsHaveDetailsOrderByBatch(BigDecimal weightBatchId);

    List<Map<String, Object>> getSteelWeightTaskResultId(BigDecimal orderId);

    //查询订单下所有计量批次
    List<Map<String, Object>> getWeightBatchIds(BigDecimal orderId);

    List<BigDecimal> getDetailIds(BigDecimal orderId);

    //获取承运商信息
    String getCarrierNameForPriceValue(String userId);

    //查询出运输订单记录下来的用户名
    String getUserName(BigDecimal orderId);

    BigDecimal getDetailAmount(BigDecimal orderId);

    int updatePlace(Map<String, Object> mapValue);

    int updatePlaceNew(Map<String, Object> mapValue);

    Map<String, Object> getsaleShippingAddressId(Map<String, Object> mapValue);

    int updateDetailsStatus(Integer detailsId);

    int getSaleOrderStatus(BigDecimal orderId);

    //获取单价
    BigDecimal getPriceIdInt(Integer placeId);

    //获取单价Id
    BigDecimal getOmsTruckOrderPriceId(BigDecimal orderId);

    //查询主键最大值
    @Select("select seq_truck_price.nextval from dual")
    BigDecimal selectMaxIdNew();

    //新增单价
    int insertPriceValue(Map<String,Object> map);

    //修改车序号单价
    int updateAsomPriceId(Map<String, Object> mapValue);

    int updateActuallyPlace(Map<String, Object> mapValue);

    int cancelMakeSure(Integer detailsId);

    int updateTruckDetails(Map<String,Object> map);

    int updateConsignee(Map<String,Object> map);

    int updateOrderPrice(@Param("orderId") BigDecimal orderId, @Param("priceId") BigDecimal priceId);

    int updateAsomPriceIdByOrderId(@Param("orderId") BigDecimal orderId, @Param("priceId") BigDecimal priceId);

    String getUserNameByUserId(String userId);

    List<BigDecimal> getOrderIds(BigDecimal orderId);

    BigDecimal getWeightBatchNetWeight(BigDecimal weightBatchId);

    BigDecimal getDetailsIdByBatchId(BigDecimal weightBatchId);

}