package com.steerinfo.dil.model;

import com.steerinfo.framework.model.IBasePO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.math.BigDecimal;
import java.util.Date;

@ApiModel(value="汽运运输订单")
public class OmstruckOrder implements IBasePO<BigDecimal> {
    /**
     * 运输订单id(ORDER_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="运输订单id",required=true)
    private BigDecimal orderId;

    /**
     * 销售订单和物资的中间表ID/发运计划ID/采购订单ID/内转物流计划ID/零星订单ID(ORDER_PLAN_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="销售订单和物资的中间表ID/发运计划ID/采购订单ID/内转物流计划ID/零星订单ID",required=false)
    private BigDecimal orderPlanId;

    /**
     * 运输订单号（WYSDD-年月日-8位数的顺序号）(ORDER_NUMBER,VARCHAR,50)
     */
    @ApiModelProperty(value="运输订单号（WYSDD-年月日-8位数的顺序号）",required=false)
    private String orderNumber;

    /**
     * 运力ID(CAPACITY_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="运力ID",required=false)
    private BigDecimal capacityId;

    /**
     * 订单下发时间(ORDER_ISSUE_TIME,TIMESTAMP,7)
     */
    @ApiModelProperty(value="订单下发时间",required=false)
    private Date orderIssueTime;

    /**
     * 司机操作订单行为：0：拒绝；1：接收(ORDER_RECEIVE_STATUS,DECIMAL,0)
     */
    @ApiModelProperty(value="司机操作订单行为：0：拒绝；1：接收",required=false)
    private BigDecimal orderReceiveStatus;

    /**
     * 司机接收/拒绝订单时间(ORDER_RECEIVE_REFUSE_TIME,TIMESTAMP,7)
     */
    @ApiModelProperty(value="司机接收/拒绝订单时间",required=false)
    private Date orderReceiveRefuseTime;

    /**
     * 通信时长=司机接单/拒绝时间-订单下发时间(ORDER_COMMUNICATION_DURATION,DECIMAL,0)
     */
    @ApiModelProperty(value="通信时长=司机接单/拒绝时间-订单下发时间",required=false)
    private BigDecimal orderCommunicationDuration;

    /**
     * 运单状态 ：0：待运输，1：执行中，2：已结束，3：未下发，4：已下发，5：已接收，6：已拒绝，7：逻辑删除 8: 关闭  9：退货关闭(ORDER_STATUS,DECIMAL,0)
     */
    @ApiModelProperty(value="运单状态 ：0：待运输，1：执行中，2：已结束，3：未下发，4：已下发，5：已接收，6：已拒绝，7：逻辑删除 8: 关闭  9：退货关闭",required=false)
    private BigDecimal orderStatus;

    /**
     * 记录创建人(INSERT_USERNAME,VARCHAR,20)
     */
    @ApiModelProperty(value="记录创建人",required=false)
    private String insertUsername;

    /**
     * 记录创建时间(INSERT_TIME,TIMESTAMP,7)
     */
    @ApiModelProperty(value="记录创建时间",required=false)
    private Date insertTime;

    /**
     * 记录修改人(UPDATE_USERNAME,VARCHAR,20)
     */
    @ApiModelProperty(value="记录修改人",required=false)
    private String updateUsername;

    /**
     * 记录修改时间(UPDATE_TIME,TIMESTAMP,7)
     */
    @ApiModelProperty(value="记录修改时间",required=false)
    private Date updateTime;

    /**
     * 记录创建或修改备注(INSERT_UPDATE_REMARK,VARCHAR,100)
     */
    @ApiModelProperty(value="记录创建或修改备注",required=false)
    private String insertUpdateRemark;

    /**
     * 逻辑删除(DELETED,DECIMAL,0)
     */
    @ApiModelProperty(value="逻辑删除",required=false)
    private BigDecimal deleted;

    /**
     * 订单类型（1：销售订单；2：焦炭发运计划；3：水渣发运计划；4：内转到异地库；5：采购辅料订单；6：采购老区燃料订单；7：采购新区燃料订单；8：采购进口矿(万州-厂内)；9：采购内转(达州-厂内)；10：采购内转(老区-厂内)； 11：厂内内转；12：零星物资进厂、13：零星物资出厂:14：钢材退货 ;15 内转精煤进厂；16 内转精煤出厂；17 采购化工材料新区订单； 18 采购化工材料老区订单）(ORDER_TYPE,DECIMAL,0)
     */
    @ApiModelProperty(value="订单类型（1：销售订单；2：焦炭发运计划；3：水渣发运计划；4：内转到异地库；5：采购辅料订单；6：采购老区燃料订单；7：采购新区燃料订单；8：采购进口矿(万州-厂内)；9：采购内转(达州-厂内)；10：采购内转(老区-厂内)； 11：厂内内转；12：零星物资进厂、13：零星物资出厂:14：钢材退货 ;15 内转精煤进厂；16 内转精煤出厂；17 采购化工材料新区订单； 18 采购化工材料老区订单）",required=false)
    private BigDecimal orderType;

    /**
     * 路段顺序号：（1：进厂 ； 2：计毛；3：卸货；4：计皮；5：出厂；6：装货）(ORDER_LINE_SEQUENCE,DECIMAL,0)
     */
    @ApiModelProperty(value="路段顺序号：（1：进厂 ； 2：计毛；3：卸货；4：计皮；5：出厂；6：装货）",required=false)
    private BigDecimal orderLineSequence;

    /**
     * 运输路线ID(LINE_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="运输路线ID",required=false)
    private BigDecimal lineId;

    /**
     * 预计进厂时间(ORDER_ENTRY_TIME,TIMESTAMP,7)
     */
    @ApiModelProperty(value="预计进厂时间",required=false)
    private Date orderEntryTime;

    /**
     * 卸货点ID(UNLOAD_POINT_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="卸货点ID",required=false)
    private BigDecimal unloadPointId;

    /**
     * 司机确认订单（1：已确认）(DRIVER_CONFIRMATION,DECIMAL,0)
     */
    @ApiModelProperty(value="司机确认订单（1：已确认）",required=false)
    private BigDecimal driverConfirmation;

    /**
     * 运输单价ID(PRICE_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="运输单价ID",required=false)
    private BigDecimal priceId;

    /**
     * 订单所属承运商ID(CARRIER_ID,DECIMAL,0)
     */
    @ApiModelProperty(value="订单所属承运商ID",required=false)
    private BigDecimal carrierId;

    /**
     *  记录下来的修改单价人
     */
    @ApiModelProperty(value="记录下来的修改单价人",required=false)
    private String recordUsername;

    public String getRecordUsername() {
        return recordUsername;
    }

    public void setRecordUsername(String recordUsername) {
        this.recordUsername = recordUsername;
    }

    private static final long serialVersionUID = 1L;

    @Override
    public BigDecimal getId() {
        return this.orderId;
    }

    @Override
    public void setId(BigDecimal orderId) {
        this.orderId = orderId;
    }

    public BigDecimal getOrderId() {
        return orderId;
    }

    public void setOrderId(BigDecimal orderId) {
        this.orderId = orderId;
    }

    public BigDecimal getOrderPlanId() {
        return orderPlanId;
    }

    public void setOrderPlanId(BigDecimal orderPlanId) {
        this.orderPlanId = orderPlanId;
    }

    public String getOrderNumber() {
        return orderNumber;
    }

    public void setOrderNumber(String orderNumber) {
        this.orderNumber = orderNumber == null ? null : orderNumber.trim();
    }

    public BigDecimal getCapacityId() {
        return capacityId;
    }

    public void setCapacityId(BigDecimal capacityId) {
        this.capacityId = capacityId;
    }

    public Date getOrderIssueTime() {
        return orderIssueTime;
    }

    public void setOrderIssueTime(Date orderIssueTime) {
        this.orderIssueTime = orderIssueTime;
    }

    public BigDecimal getOrderReceiveStatus() {
        return orderReceiveStatus;
    }

    public void setOrderReceiveStatus(BigDecimal orderReceiveStatus) {
        this.orderReceiveStatus = orderReceiveStatus;
    }

    public Date getOrderReceiveRefuseTime() {
        return orderReceiveRefuseTime;
    }

    public void setOrderReceiveRefuseTime(Date orderReceiveRefuseTime) {
        this.orderReceiveRefuseTime = orderReceiveRefuseTime;
    }

    public BigDecimal getOrderCommunicationDuration() {
        return orderCommunicationDuration;
    }

    public void setOrderCommunicationDuration(BigDecimal orderCommunicationDuration) {
        this.orderCommunicationDuration = orderCommunicationDuration;
    }

    public BigDecimal getOrderStatus() {
        return orderStatus;
    }

    public void setOrderStatus(BigDecimal orderStatus) {
        this.orderStatus = orderStatus;
    }

    public String getInsertUsername() {
        return insertUsername;
    }

    public void setInsertUsername(String insertUsername) {
        this.insertUsername = insertUsername == null ? null : insertUsername.trim();
    }

    public Date getInsertTime() {
        return insertTime;
    }

    public void setInsertTime(Date insertTime) {
        this.insertTime = insertTime;
    }

    public String getUpdateUsername() {
        return updateUsername;
    }

    public void setUpdateUsername(String updateUsername) {
        this.updateUsername = updateUsername == null ? null : updateUsername.trim();
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    public String getInsertUpdateRemark() {
        return insertUpdateRemark;
    }

    public void setInsertUpdateRemark(String insertUpdateRemark) {
        this.insertUpdateRemark = insertUpdateRemark == null ? null : insertUpdateRemark.trim();
    }

    public BigDecimal getDeleted() {
        return deleted;
    }

    public void setDeleted(BigDecimal deleted) {
        this.deleted = deleted;
    }

    public BigDecimal getOrderType() {
        return orderType;
    }

    public void setOrderType(BigDecimal orderType) {
        this.orderType = orderType;
    }

    public BigDecimal getOrderLineSequence() {
        return orderLineSequence;
    }

    public void setOrderLineSequence(BigDecimal orderLineSequence) {
        this.orderLineSequence = orderLineSequence;
    }

    public BigDecimal getLineId() {
        return lineId;
    }

    public void setLineId(BigDecimal lineId) {
        this.lineId = lineId;
    }

    public Date getOrderEntryTime() {
        return orderEntryTime;
    }

    public void setOrderEntryTime(Date orderEntryTime) {
        this.orderEntryTime = orderEntryTime;
    }

    public BigDecimal getUnloadPointId() {
        return unloadPointId;
    }

    public void setUnloadPointId(BigDecimal unloadPointId) {
        this.unloadPointId = unloadPointId;
    }

    public BigDecimal getDriverConfirmation() {
        return driverConfirmation;
    }

    public void setDriverConfirmation(BigDecimal driverConfirmation) {
        this.driverConfirmation = driverConfirmation;
    }

    public BigDecimal getPriceId() {
        return priceId;
    }

    public void setPriceId(BigDecimal priceId) {
        this.priceId = priceId;
    }

    public BigDecimal getCarrierId() {
        return carrierId;
    }

    public void setCarrierId(BigDecimal carrierId) {
        this.carrierId = carrierId;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getClass().getSimpleName());
        sb.append(" [");
        sb.append("Hash = ").append(hashCode());
        sb.append(", orderId=").append(orderId);
        sb.append(", orderPlanId=").append(orderPlanId);
        sb.append(", orderNumber=").append(orderNumber);
        sb.append(", capacityId=").append(capacityId);
        sb.append(", orderIssueTime=").append(orderIssueTime);
        sb.append(", orderReceiveStatus=").append(orderReceiveStatus);
        sb.append(", orderReceiveRefuseTime=").append(orderReceiveRefuseTime);
        sb.append(", orderCommunicationDuration=").append(orderCommunicationDuration);
        sb.append(", orderStatus=").append(orderStatus);
        sb.append(", insertUsername=").append(insertUsername);
        sb.append(", insertTime=").append(insertTime);
        sb.append(", updateUsername=").append(updateUsername);
        sb.append(", updateTime=").append(updateTime);
        sb.append(", insertUpdateRemark=").append(insertUpdateRemark);
        sb.append(", deleted=").append(deleted);
        sb.append(", orderType=").append(orderType);
        sb.append(", orderLineSequence=").append(orderLineSequence);
        sb.append(", lineId=").append(lineId);
        sb.append(", orderEntryTime=").append(orderEntryTime);
        sb.append(", unloadPointId=").append(unloadPointId);
        sb.append(", driverConfirmation=").append(driverConfirmation);
        sb.append(", priceId=").append(priceId);
        sb.append(", carrierId=").append(carrierId);
        sb.append(", serialVersionUID=").append(serialVersionUID);
        sb.append("]");
        return sb.toString();
    }
}